#!/usr/bin/env python

# Author: DuSu

import argparse
import socket
import string
import sys
from Crypto.Cipher import Blowfish
from Crypto.Hash import MD5

# defaults (for old models)
HOST = '192.168.1.1'
PORT = 23 # default telnet port
USER = 'Gearguy'
PASS = 'Geardog'
WORD_SIZE = 4 # most if not all routers should have 32 bits cpu
ENDIAN = 'little' # 'little' or 'big'
KEY_PREFIX = 'AMBIT_TELNET_ENABLE+'
MAC_BUF_LEN = 0x10
USER_BUF_LEN = 0x10
# The blowfish algo accepts keys of max size 448 bits, i.e., 56 bytes.
# Since len(KEY_PREFIX) is 20, we are left with 36 = 0x20 bytes for the
# password, even if the full payload as more room. The password should thus be
# at maximum 36 = 0x24 long (for the max size, the trailing null byte will
# come from the null buffer that is after the password
PASSWD_BUF_LEN = 0x24
# The full payload, not counting the md5 hash MUST be 0x70 long otherwise
# the "exploit" will fail
PAYLOAD_BUF_LEN = 0x70

def good_mac(mac):
	"""
	Checks and formats the mac address so that it looks like
	AABBCCDDEEFF for inputs
	AA:BB:CC:DD:EE:FF, AABBCCDDEEFF, aa:bb:cc:dd:ee:ff, aabbccddeeff
	"""
	rmac = mac.replace(':', '')
	rmac = rmac.upper()
	if len(rmac) != 12 or not(all(c in string.hexdigits for c in rmac)):
		print('{} is not a valid mac address'.format(mac))
		sys.exit(0)
	return rmac

def pad(string, size):
	"""
	Returns string, possibly padded with null bytes so that the final length
	is a multiple of size 
	"""
	length = len(string)
	if length % size == 0:
		return string
	else:
		length = (1 + length // size) * size
		return string.ljust(length, '\x00')

def swap_endianness(string, word_size = WORD_SIZE):
	"""
	Reverses characters of a string, word by word
	"""
	string = pad(string, word_size)
	words = map(''.join, zip(*[iter(string)] * word_size))
	return ''.join([w[::-1] for w in words])

def enabler_payload(mac, user, passwd, big_endian):
	"""
	Builds the payload that will enable telnet or parser, depending on
	credentials
	"""
	mac = good_mac(mac)
	if len(user) >= USER_BUF_LEN:
		print('username {} is too long (maximum length is {})'.format(user, USER_BUF_LEN - 1))
		sys.exit(0)
	if len(passwd) > PASSWD_BUF_LEN:
		print('password {} is too long (maximum length is {})'.format(user, PASSWD_BUF_LEN))
		sys.exit(0)
	payload = ''
	payload += pad(mac, MAC_BUF_LEN)
	payload += pad(user, USER_BUF_LEN)
	payload += passwd
	payload = pad(payload, PAYLOAD_BUF_LEN)
	md5hash = MD5.new(payload)
	payload = md5hash.digest() + payload # digest is 0x10 long so no padding required
	key = KEY_PREFIX + passwd
	cipher = Blowfish.new(key, 1)
	if not big_endian:
		payload = swap_endianness(payload)
	payload = cipher.encrypt(payload)
	if not big_endian:
		payload = swap_endianness(payload)
	return payload

def send(host, port, socket_type, payload):
	"""
	Sends the payload, no check is performed
	"""
	s = socket.socket(socket.AF_INET, socket_type)
	s.connect((host, port))
	s.send(payload)
	s.close()

def main():
	"""
	Parses arguments, creates the payload and sends it
	"""
	parser = argparse.ArgumentParser(
		description = 'Telnetable for Netgear rpouters',
		usage = './%(prog)s <options>')
	parser.add_argument('-i', '--host',
		default = HOST,
		action = 'store',
		help = 'ip address (lan) of the router (default: %(default)s)')
	parser.add_argument('-m', '--mac',
		action = 'store',
		required = True,
		help = 'mac address of the router')
	parser.add_argument('-u', '--user',
		default = USER,
		action = 'store',
		help = 'user name (default: %(default)s)')
	parser.add_argument('-p', '--passwd',
		default = PASS,
		action = 'store',
		help = 'password (default: %(default)s)')
	parser.add_argument('-U', '--udp',
		default = False,
		action = 'store_true',
		help = 'use UDP protocol instead of TCP protocol')
	parser.add_argument('-b', '--big',
		default = False,
		action = 'store_true',
		help = 'use this if the router is big endian')
	parser.add_argument('-P', '--port',
		default = PORT,
		action = 'store',
		type = int,
		help = 'port (default: %(default)s)')
	args = parser.parse_args()	
	if args.udp:
		socket_type = socket.SOCK_DGRAM
	else:
		socket_type = socket.SOCK_STREAM
	payload = enabler_payload(args.mac, args.user, args.passwd, args.big)
	send(args.host, args.port, socket_type, payload)

if __name__ == '__main__':
	main()
